@@ -13,8 +13,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from http import HTTPStatus
 import pecan
-from six.moves import http_client
 import wsme
 from wsme import types as wtypes
 
@@ -23,11 +23,13 @@ from oslo_log import log
 from cyborg.api.controllers import base
 from cyborg.api.controllers import link
 from cyborg.api.controllers import types
+from cyborg.api.controllers.v2 import utils
+from cyborg.api.controllers.v2 import versions
 from cyborg.api import expose
+from cyborg.common import authorize_wsgi
 from cyborg.common import constants
 from cyborg.common import exception
 from cyborg.common.i18n import _
-from cyborg.common import policy
 from cyborg import objects
 
 LOG = log.getLogger(__name__)
@@ -40,7 +42,7 @@ class ARQ(base.APIBase):
     between the internal object model and the API representation.
     """
     uuid = types.uuid
-    """The UUID of the device profile"""
+    """The UUID of the ARQ"""
 
     state = wtypes.text  # obvious meanings
     device_profile_name = wtypes.text
@@ -54,6 +56,8 @@ class ARQ(base.APIBase):
 
     instance_uuid = wtypes.text
     """The UUID of the instance associated with this ARQ, if any"""
+    project_id = wtypes.text
+    """The UUID of the instance project_id associated with this ARQ, if any"""
 
     attach_handle_type = wtypes.text
     attach_handle_info = {wtypes.text: wtypes.text}
@@ -95,25 +99,13 @@ class ARQCollection(base.APIBase):
 class ARQsController(base.CyborgController):
     """REST controller for ARQs.
 
-       For the relationship betweens ARQs and device profiles, see
+       For the relationship between ARQs and device profiles, see
        nova/nova/accelerator/cyborg.py.
     """
 
-    def _get_devprof(self, context, devprof_name):
-        """Get the contents of a device profile.
-           Since this is just a read, it is ok for the API layer
-           to do this, instead of the conductor.
-        """
-        try:
-            obj_devprof = objects.DeviceProfile.get_by_name(context,
-                                                            devprof_name)
-            return obj_devprof
-        except Exception:
-            return None
-
-    @policy.authorize_wsgi("cyborg:arq", "create", False)
+    @authorize_wsgi.authorize_wsgi("cyborg:arq", "create", False)
     @expose.expose(ARQCollection, body=types.jsontype,
-                   status_code=http_client.CREATED)
+                   status_code=HTTPStatus.CREATED)
     def post(self, req):
         """Create one or more ARQs for a single device profile.
            Request body:
@@ -127,14 +119,16 @@ class ARQsController(base.CyborgController):
         """
         LOG.info("[arq] post req = (%s)", req)
         context = pecan.request.context
-        devprof = None
         dp_name = req.get('device_profile_name')
         if dp_name is not None:
-            devprof = self._get_devprof(context, dp_name)
-            if devprof is None:
+            try:
+                devprof = objects.DeviceProfile.get_by_name(context, dp_name)
+            except exception.ResourceNotFound:
                 raise exception.ResourceNotFound(
                     resource='Device Profile',
                     msg='with name=%s' % dp_name)
+            except Exception as e:
+                raise e
         else:
             raise exception.DeviceProfileNameNeeded()
         LOG.info('[arqs] post. device profile name=%s', dp_name)
@@ -156,8 +150,8 @@ class ARQsController(base.CyborgController):
                 obj_arq = objects.ARQ(context, **arq_fields)
                 extarq_fields = {'arq': obj_arq}
                 obj_extarq = objects.ExtARQ(context, **extarq_fields)
-                # TODO(Sundar) The conductor must do all db writes
-                new_extarq = obj_extarq.create(context, devprof.id)
+                new_extarq = pecan.request.conductor_api.arq_create(
+                    context, obj_extarq, devprof.id)
                 extarq_list.append(new_extarq)
 
         ret = ARQCollection.convert_with_links(
@@ -165,7 +159,7 @@ class ARQsController(base.CyborgController):
         LOG.info('[arqs] post returned: %s', ret)
         return ret
 
-    @policy.authorize_wsgi("cyborg:arq", "get_one")
+    @authorize_wsgi.authorize_wsgi("cyborg:arq", "get_one")
     @expose.expose(ARQ, wtypes.text)
     def get_one(self, uuid):
         """Get a single ARQ by UUID."""
@@ -173,7 +167,7 @@ class ARQsController(base.CyborgController):
         extarq = objects.ExtARQ.get(context, uuid)
         return ARQ.convert_with_links(extarq.arq)
 
-    @policy.authorize_wsgi("cyborg:arq", "get_all", False)
+    @authorize_wsgi.authorize_wsgi("cyborg:arq", "get_all", False)
     @expose.expose(ARQCollection, wtypes.text, types.uuid)
     def get_all(self, bind_state=None, instance=None):
         """Retrieve a list of arqs."""
@@ -187,9 +181,9 @@ class ARQsController(base.CyborgController):
         arqs = [extarq.arq for extarq in extarqs]
         # TODO(Sundar): Optimize by doing the filtering in the db layer
         # Apply instance filter before state filter.
-        if bind_state:
-            if bind_state != 'resolved':
-                raise exception.ARQInvalidState(state=bind_state)
+        if bind_state and bind_state != 'resolved':
+            raise exception.ARQBadState(
+                state=bind_state, uuid=None, expected=['resolved'])
         if instance:
             new_arqs = [arq for arq in arqs
                         if arq['instance_uuid'] == instance]
@@ -203,7 +197,7 @@ class ARQsController(base.CyborgController):
                                     'resolved', instance)
                         return wsme.api.Response(
                             None,
-                            status_code=http_client.LOCKED)
+                            status_code=HTTPStatus.LOCKED)
         elif bind_state:
             arqs = [arq for arq in arqs
                     if arq['state'] in valid_bind_states]
@@ -212,9 +206,9 @@ class ARQsController(base.CyborgController):
         LOG.info('[arqs:get_all] Returned: %s', ret)
         return ret
 
-    @policy.authorize_wsgi("cyborg:arq", "delete", False)
+    @authorize_wsgi.authorize_wsgi("cyborg:arq", "delete", False)
     @expose.expose(None, wtypes.text, wtypes.text,
-                   status_code=http_client.NO_CONTENT)
+                   status_code=HTTPStatus.NO_CONTENT)
     def delete(self, arqs=None, instance=None):
         """Delete one or more ARQS.
 
@@ -230,21 +224,21 @@ class ARQsController(base.CyborgController):
         an error. Nova uses the second form: so repeated calls do not cause
         issues.
 
-        :param arq: List of ARQ UUIDs
+        :param arqs: List of ARQ UUIDs
         :param instance: UUID of instance whose ARQs need to be deleted
         """
         context = pecan.request.context
-        if (arqs and instance) or ((not arqs) and (not instance)):
+        if (arqs and instance) or (not arqs and not instance):
             raise exception.ObjectActionError(
                 action='delete',
                 reason='Provide either an ARQ uuid list or an instance UUID')
         elif arqs:
             LOG.info("[arqs] delete. arqs=(%s)", arqs)
-            arqlist = arqs.split(',')
-            objects.ExtARQ.delete_by_uuid(context, arqlist)
+            pecan.request.conductor_api.arq_delete_by_uuid(context, arqs)
         else:  # instance is not None
             LOG.info("[arqs] delete. instance=(%s)", instance)
-            objects.ExtARQ.delete_by_instance(context, instance)
+            pecan.request.conductor_api.arq_delete_by_instance_uuid(
+                context, instance)
 
     def _validate_arq_patch(self, patch):
         """Validate a single patch for an ARQ.
@@ -257,6 +251,8 @@ class ARQsController(base.CyborgController):
         valid_fields = {'hostname': None,
                         'device_rp_uuid': None,
                         'instance_uuid': None}
+        if utils.allow_project_id():
+            valid_fields['project_id'] = None
         if ((not all(p['op'] == 'add' for p in patch)) and
            (not all(p['op'] == 'remove' for p in patch))):
             raise exception.PatchError(
@@ -264,6 +260,12 @@ class ARQsController(base.CyborgController):
 
         for p in patch:
             path = p['path'].lstrip('/')
+            if path == 'project_id' and not utils.allow_project_id():
+                raise exception.NotAcceptable(_(
+                    "Request not acceptable. The minimal required API "
+                    "version should be %(base)s.%(opr)s") %
+                    {'base': versions.BASE_VERSION,
+                     'opr': versions.MINOR_1_PROJECT_ID})
             if path not in valid_fields.keys():
                 reason = 'Invalid path in patch {}'.format(p['path'])
                 raise exception.PatchError(reason=reason)
@@ -292,9 +294,9 @@ class ARQsController(base.CyborgController):
             reason = msg.format(instance_uuid)
             raise exception.PatchError(reason=reason)
 
-    @policy.authorize_wsgi("cyborg:arq", "update", False)
+    @authorize_wsgi.authorize_wsgi("cyborg:arq", "update", False)
     @expose.expose(None, body=types.jsontype,
-                   status_code=http_client.ACCEPTED)
+                   status_code=HTTPStatus.ACCEPTED)
     def patch(self, patch_list):
         """Bind/Unbind one or more ARQs.
 
@@ -306,10 +308,11 @@ class ARQsController(base.CyborgController):
                 {"path": "/hostname", "op": ADD/RM, "value": "..."},
                 {"path": "/device_rp_uuid", "op": ADD/RM, "value": "..."},
                 {"path": "/instance_uuid", "op": ADD/RM, "value": "..."},
+                {"path": "/project_id", "op": ADD/RM, "value": "..."},
                ],
              "$arq_uuid": [...]
             }
-            In particular, all and only these 3 fields must be present,
+            In particular, all and only these 4 fields must be present,
             and only 'add' or 'remove' ops are allowed.
         """
         LOG.info('[arqs] patch. list=(%s)', patch_list)
@@ -331,5 +334,5 @@ class ARQsController(base.CyborgController):
         if patch[0]['op'] == 'add':
             self._check_if_already_bound(context, valid_fields)
 
-        # TODO(Sundar) Defer to conductor and do all concurently.
-        objects.ExtARQ.apply_patch(context, patch_list, valid_fields)
+        pecan.request.conductor_api.arq_apply_patch(
+            context, patch_list, valid_fields)
